@@ -47,6 +47,8 @@
 #include <camel/camel-folder.h>
 #include <camel/camel-stream-mem.h>
 #include <camel/camel-mime-message.h>
+#include <camel/camel-file-utils.h>
+#include <camel/camel-stream-fs.h>
 
 #include "mail/mail-tools.h"
 
@@ -60,6 +62,7 @@
 #include "recur-comp.h"
 #include "comp-editor.h"
 #include "../e-cal-popup.h"
+#include "cal-attachment-select-file.h"
 
 #include "e-attachment-bar.h"
 #include "misc/e-expander.h"
@@ -611,6 +614,7 @@ get_attachment_list (CompEditor *editor)
 		CamelStream *stream;
 		char *attach_file_url;
 		char *safe_fname;
+		char *filename;
 	
 		wrapper = camel_medium_get_content_object (CAMEL_MEDIUM (p->data));
 			
@@ -618,30 +622,35 @@ get_attachment_list (CompEditor *editor)
 		 * as a mime part file into the directory denoting the
 		 * calendar source */
 		safe_fname = camel_file_util_safe_filename(camel_mime_part_get_filename ((CamelMimePart *)p->data));
-		attach_file_url = g_strconcat (local_store,
-					       comp_uid,  "-",
-			 		       safe_fname,
-		   			       NULL); 
+
+		filename = g_strdup_printf ("%s-%s", comp_uid, safe_fname);
+
+		attach_file_url = g_build_path ("/", local_store, filename, NULL);
+
+		g_free (filename);
 		g_free (safe_fname);
 
-		stream = camel_stream_fs_new_with_name(attach_file_url+7, O_RDWR|O_CREAT|O_TRUNC, 0600);
-		if (!stream) {
-			/* TODO handle error conditions */
-			g_message ("DEBUG: could not open the file to write\n");
-			g_free (attach_file_url);
-			continue;
-		}
+		/* do not overwrite existing files, this will result in truncation */
+		if (!g_file_test (attach_file_url+7, G_FILE_TEST_EXISTS)) {
+			stream = camel_stream_fs_new_with_name((const char *) attach_file_url+7, O_RDWR|O_CREAT|O_TRUNC, 0600);
+			if (!stream) {
+				/* TODO handle error conditions */
+				g_message ("DEBUG: could not open the file to write\n");
+				g_free (attach_file_url);
+				continue;
+			}
 		
-		if (camel_data_wrapper_decode_to_stream (wrapper, (CamelStream *) stream) == -1) {
-			g_free (attach_file_url);
+			if (camel_data_wrapper_decode_to_stream (wrapper, (CamelStream *) stream) == -1) {
+				g_free (attach_file_url);
+				camel_stream_close (stream);
+				camel_object_unref (stream);
+				g_message ("DEBUG: could not write to file\n");
+			}
+
 			camel_stream_close (stream);
 			camel_object_unref (stream);
-			g_message ("DEBUG: could not write to file\n");
 		}
 
-		camel_stream_close (stream);
-		camel_object_unref (stream);
-
 		list = g_slist_append (list, g_strdup (attach_file_url));
 		g_free (attach_file_url);
 	}
@@ -810,7 +819,16 @@ save_comp_with_send (CompEditor *editor)
 	
 	send = priv->changed && priv->needs_send;
 	delegate = priv->flags & COMP_EDITOR_DELEGATE;
-
+	
+	if (delegate) {
+		icalcomponent *icalcomp = e_cal_component_get_icalcomponent (priv->comp);
+		icalproperty *icalprop;
+		 
+		icalprop = icalproperty_new_x ("1");
+		icalproperty_set_x_name (icalprop, "X-EVOLUTION-DELEGATED");
+		icalcomponent_add_property (icalcomp, icalprop);
+	}
+	
 	if (!save_comp (editor))
 		return FALSE;
 
@@ -988,7 +1006,8 @@ attachment_bar_icon_clicked_cb (EAttachmentBar *bar, GdkEvent *event, CompEditor
 		if (p) {
 			EAttachment *attachment;
 			GSList *list;
-			const char *comp_uid= NULL;
+			const char *comp_uid = NULL;
+			char *filename = NULL;
 			const char *local_store = e_cal_get_local_attachment_store (editor->priv->client);
 
 			e_cal_component_get_uid (editor->priv->comp, &comp_uid);			
@@ -996,19 +1015,21 @@ attachment_bar_icon_clicked_cb (EAttachmentBar *bar, GdkEvent *event, CompEditor
 			list = e_attachment_bar_get_attachment (bar, num);
 			attachment = list->data;
 			g_slist_free (list);
+
+			filename = g_strdup_printf ("%s-%s",
+						    comp_uid,
+						    camel_mime_part_get_filename(attachment->body));
+
+			attach_file_url = g_build_path ("/", local_store, filename, NULL);
 			
-			attach_file_url = g_strconcat (local_store,
-							comp_uid,
-							"-",
-							camel_mime_part_get_filename(attachment->body),
-							NULL);
-						
 			/* launch the url now */
 			/* TODO should send GError and handle error conditions
 			 * here */
 			gnome_url_show (attach_file_url, &error);
 			if (error)
-				g_message ("DEBUG: Launch failed :(\n");
+				g_message ("DEBUG: gnome_url_show(%s) failed\n", attach_file_url);
+
+			g_free (filename);
 			g_free (attach_file_url); }
 		return TRUE;
 	} else 
@@ -1056,10 +1077,10 @@ cab_remove(EPopup *ep, EPopupItem *item, void *data)
 
 /* Popup menu handling.  */
 static EPopupItem cab_popups[] = {
-	{ E_POPUP_ITEM, "10.attach", N_("_Remove"), cab_remove, NULL, GTK_STOCK_REMOVE, E_CAL_POPUP_ATTACHMENTS_MANY },
+	{ E_POPUP_ITEM, "10.attach", N_("_Remove"), cab_remove, NULL, GTK_STOCK_REMOVE, E_CAL_POPUP_ATTACHMENTS_MANY | E_CAL_POPUP_ATTACHMENTS_MODIFY },
 	{ E_POPUP_ITEM, "20.attach", N_("_Properties"), cab_properties, NULL, GTK_STOCK_PROPERTIES, E_CAL_POPUP_ATTACHMENTS_ONE },
 	{ E_POPUP_BAR, "30.attach.00", NULL, NULL, NULL, NULL, E_CAL_POPUP_ATTACHMENTS_MANY|E_CAL_POPUP_ATTACHMENTS_ONE },
-	{ E_POPUP_ITEM, "30.attach.01", N_("_Add attachment..."), cab_add, NULL, GTK_STOCK_ADD, 0 },
+	{ E_POPUP_ITEM, "30.attach.01", N_("_Add attachment..."), cab_add, NULL, GTK_STOCK_ADD, E_CAL_POPUP_ATTACHMENTS_MODIFY },
 };
 
 static void
@@ -1100,6 +1121,7 @@ cab_popup(EAttachmentBar *bar, GdkEventButton *event, int id)
 	ECalPopup *ecp;
 	ECalPopupTargetAttachments *t;
 	GtkMenu *menu;
+       CompEditor *editor = COMP_EDITOR (gtk_widget_get_toplevel (GTK_WIDGET (bar)));
 
         attachments = e_attachment_bar_get_attachment(bar, id);
 
@@ -1115,7 +1137,7 @@ cab_popup(EAttachmentBar *bar, GdkEventButton *event, int id)
 	 */
 	ecp = e_cal_popup_new("org.gnome.evolution.calendar.attachmentbar.popup");
 	e_popup_add_items((EPopup *)ecp, menus, NULL, cab_popups_free, bar);
-	t = e_cal_popup_target_new_attachments(ecp, attachments);
+	t = e_cal_popup_target_new_attachments(ecp, editor, attachments);
 	t->target.widget = (GtkWidget *)bar;
 	menu = e_popup_create_menu_once((EPopup *)ecp, (EPopupTarget *)t, 0);
 
@@ -1141,7 +1163,6 @@ button_press_event (GtkWidget *widget, GdkEventButton *event)
 	EAttachmentBar *bar = (EAttachmentBar *)widget;
 	GnomeIconList *icon_list = GNOME_ICON_LIST(widget);
 	int icon_number;
-	printf("dd\n");
 	if (event->button != 3)
 		return FALSE;
 	
@@ -1254,19 +1275,6 @@ setup_widgets (CompEditor *editor)
 	
 }
 
-void
-comp_editor_sensitize_attachment_bar (CompEditor *editor, gboolean  set)
-{
-	CompEditorPrivate *priv;
-
-	g_return_if_fail (IS_COMP_EDITOR (editor));
-
-	priv = editor->priv;
-	
-	gtk_widget_set_sensitive (GTK_WIDGET (priv->attachment_bar),  set);
-	gtk_widget_set_sensitive (GTK_WIDGET (priv->attachment_scrolled_window),  set);
-}
-	
 /* Object initialization function for the calendar component editor */
 static void
 comp_editor_init (CompEditor *editor)
@@ -2005,7 +2013,7 @@ set_attachment_list (CompEditor *editor, GSList *attach_list)
 
 	e_cal_component_get_uid (editor->priv->comp, &comp_uid);
 
-	if (e_attachment_bar_get_num_attachments (editor->priv->attachment_bar)) {
+	if (e_attachment_bar_get_num_attachments (E_ATTACHMENT_BAR (editor->priv->attachment_bar))) {
 		/* To prevent repopulating the
 		 * bar due to redraw functions in fill_widget. 
 		 * Assumes it can be set only once.
@@ -2071,9 +2079,13 @@ set_attachment_list (CompEditor *editor, GSList *attach_list)
 	
 		camel_mime_part_set_disposition (part, "attachment");
 
-		camel_mime_part_set_filename (part,
-					      attach_filename + strlen (local_store)+ 
-						strlen (comp_uid) + 1); 
+		char *ptr = strstr (attach_filename, comp_uid);
+		if (ptr) {
+			ptr += strlen(comp_uid);
+			if (*ptr++ == '-')
+				camel_mime_part_set_filename (part, ptr); 
+		}
+
 		e_attachment_bar_attach_mime_part ((EAttachmentBar *) editor->priv->attachment_bar, part);
 
 		camel_object_unref (part);
@@ -2402,13 +2414,13 @@ comp_editor_get_mime_attach_list (CompEditor *editor)
 	GSList *attach_list = NULL, *l, *parts;
 
 	/* TODO assert sanity of bar */
-	parts = e_attachment_bar_get_parts (editor->priv->attachment_bar);
+	parts = e_attachment_bar_get_parts (E_ATTACHMENT_BAR (editor->priv->attachment_bar));
 	for (l = parts; l ; l = l->next) {
 		
 		CamelDataWrapper *wrapper;
 		CamelStreamMem *mstream;
 		unsigned char *buffer = NULL;
-		char *desc;
+		const char *desc;
 
 		cal_mime_attach = g_malloc0 (sizeof (struct CalMimeAttach));
 		wrapper = camel_medium_get_content_object (CAMEL_MEDIUM (l->data));
@@ -2496,7 +2508,6 @@ comp_editor_notify_client_changed (CompEditor *editor, ECal *client)
 
 	if (!e_cal_is_read_only (client, &read_only, NULL))
 		read_only = TRUE;
-	comp_editor_sensitize_attachment_bar (editor, !read_only);
 	gtk_dialog_set_response_sensitive (GTK_DIALOG (editor), GTK_RESPONSE_OK, !read_only);
 }
 
