@@ -43,7 +43,7 @@
 #include <glade/glade.h>
 #include <e-util/e-dialog-widgets.h>
 #include <e-util/e-time-utils.h>
-#include <widgets/misc/e-dateedit.h>
+#include <misc/e-dateedit.h>
 #include <libecal/e-cal-recur.h>
 #include <libecal/e-cal-time-util.h>
 #include "../calendar-config.h"
@@ -470,10 +470,13 @@ static void
 sensitize_recur_widgets (RecurrencePage *rpage)
 {
 	RecurrencePagePrivate *priv;
-	gboolean recurs;
+	gboolean recurs, sens = TRUE;
 	GtkWidget *label;
 
 	priv = rpage->priv;
+	
+	if (COMP_EDITOR_PAGE (rpage)->flags & COMP_EDITOR_PAGE_MEETING)
+		sens = COMP_EDITOR_PAGE (rpage)->flags & COMP_EDITOR_PAGE_USER_ORG;
 
 	recurs = e_dialog_toggle_get (priv->recurs);
 	
@@ -481,7 +484,7 @@ sensitize_recur_widgets (RecurrencePage *rpage)
 	if (e_cal_component_is_instance (priv->comp))
 		gtk_widget_set_sensitive (priv->preview_calendar, FALSE);
 	else
-		gtk_widget_set_sensitive (priv->preview_calendar, TRUE);
+		gtk_widget_set_sensitive (priv->preview_calendar, TRUE && sens);
 
 	if (GTK_BIN (priv->custom_warning_bin)->child)
 		gtk_widget_destroy (GTK_BIN (priv->custom_warning_bin)->child);
@@ -497,7 +500,7 @@ sensitize_recur_widgets (RecurrencePage *rpage)
 				   label);
 		gtk_widget_show_all (priv->custom_warning_bin);
 	} else if (recurs) {
-		gtk_widget_set_sensitive (priv->params, TRUE);
+		gtk_widget_set_sensitive (priv->params, sens);
 		gtk_widget_show (priv->params);
 		gtk_widget_hide (priv->custom_warning_bin);
 	} else {
@@ -510,14 +513,16 @@ sensitize_recur_widgets (RecurrencePage *rpage)
 static void
 sensitize_buttons (RecurrencePage *rpage)
 {
-	gboolean read_only;
+	gboolean read_only, sensitize = TRUE;
 	gint selected_rows;
 	RecurrencePagePrivate *priv;
 	icalcomponent *icalcomp;
 	const char *uid;
 
 	priv = rpage->priv;
-
+	if (COMP_EDITOR_PAGE (rpage)->flags & COMP_EDITOR_PAGE_MEETING)
+		sensitize = COMP_EDITOR_PAGE (rpage)->flags & COMP_EDITOR_PAGE_USER_ORG;
+	
 	selected_rows = gtk_tree_selection_count_selected_rows (
 		gtk_tree_view_get_selection (GTK_TREE_VIEW (priv->exception_list)));
 
@@ -531,6 +536,19 @@ sensitize_buttons (RecurrencePage *rpage)
 			read_only = TRUE;
 			icalcomponent_free (icalcomp);
 		}
+
+		if (!read_only) {
+			GList *list;
+
+			/* see if we have detached instances */
+			if (e_cal_get_objects_for_uid (COMP_EDITOR_PAGE (rpage)->client, uid, &list, NULL)) {
+				if (list && g_list_length (list) > 1)
+					read_only = TRUE;
+
+				g_list_foreach (list, (GFunc) g_object_unref, NULL);
+				g_list_free (list);
+			}
+		}
 	}
 
 	if (!read_only)
@@ -538,10 +556,10 @@ sensitize_buttons (RecurrencePage *rpage)
 	else
 		gtk_widget_set_sensitive (priv->params, FALSE);
 
-	gtk_widget_set_sensitive (priv->recurs, !read_only);
-	gtk_widget_set_sensitive (priv->exception_add, !read_only && e_cal_component_has_recurrences (priv->comp));
-	gtk_widget_set_sensitive (priv->exception_modify, !read_only && selected_rows > 0);
-	gtk_widget_set_sensitive (priv->exception_delete, !read_only && selected_rows > 0);
+	gtk_widget_set_sensitive (priv->recurs, !read_only && sensitize);
+	gtk_widget_set_sensitive (priv->exception_add, !read_only && e_cal_component_has_recurrences (priv->comp) && sensitize);
+	gtk_widget_set_sensitive (priv->exception_modify, !read_only && selected_rows > 0 && sensitize);
+	gtk_widget_set_sensitive (priv->exception_delete, !read_only && selected_rows > 0 && sensitize);
 }
 
 #if 0
@@ -921,7 +939,6 @@ make_weekly_special (RecurrencePage *rpage)
 
 	weekday_picker_set_week_start_day (wp, calendar_config_get_week_start_day ());
 	weekday_picker_set_days (wp, priv->weekday_day_mask);
-	weekday_picker_set_blocked_days (wp, priv->weekday_blocked_day_mask);
 
 	g_signal_connect((wp), "changed",
 			    G_CALLBACK (weekday_picker_changed_cb),
@@ -1278,6 +1295,7 @@ make_ending_until_special (RecurrencePage *rpage)
 {
 	RecurrencePagePrivate *priv;
 	EDateEdit *de;
+	ECalComponentDateTime dt_start;
 
 	priv = rpage->priv;
 
@@ -1296,9 +1314,11 @@ make_ending_until_special (RecurrencePage *rpage)
 
 	/* Set the value */
 
-	e_date_edit_set_date (de, priv->ending_date_tt.year,
-			      priv->ending_date_tt.month,
-			      priv->ending_date_tt.day);
+	e_cal_component_get_dtstart (priv->comp, &dt_start);
+	/* Setting the default until time to 2 weeks */
+	icaltime_adjust (dt_start.value, 14, 0, 0, 0); 
+	e_date_edit_set_date (de, dt_start.value->year, dt_start.value->month, dt_start.value->day);
+	e_cal_component_free_datetime (&dt_start);
 
 	g_signal_connect((de), "changed",
 			    G_CALLBACK (ending_until_changed_cb), rpage);
@@ -1505,6 +1525,9 @@ recurrence_page_fill_widgets (CompEditorPage *page, ECalComponent *comp)
 
 	priv->comp = e_cal_component_clone (comp);
 
+	if (!e_cal_component_has_organizer (comp))
+		 page->flags |= COMP_EDITOR_PAGE_USER_ORG;
+	
 	/* Don't send off changes during this time */
 	priv->updating = TRUE;
 
