@@ -47,8 +47,8 @@
 #include "dialogs/comp-editor.h"
 #include "dialogs/copy-source-dialog.h"
 #include "dialogs/task-editor.h"
-#include "widgets/misc/e-info-label.h"
-#include "widgets/misc/e-error.h"
+#include "misc/e-info-label.h"
+#include "e-util/e-error.h"
 #include "e-util/e-icon-factory.h"
 
 #define CREATE_TASK_ID               "task"
@@ -196,6 +196,17 @@ ensure_sources (TasksComponent *component)
 		ESource *source = e_source_new (_("Personal"), PERSONAL_RELATIVE_URI);
 		e_source_group_add_source (on_this_computer, source, -1);
 
+		if (!calendar_config_get_primary_tasks () && !calendar_config_get_tasks_selected ()) {
+			GSList selected;
+
+			calendar_config_set_primary_tasks (e_source_peek_uid (source));
+
+			selected.data = (gpointer)e_source_peek_uid (source);
+			selected.next = NULL;
+			calendar_config_set_tasks_selected (&selected);
+		}
+
+		e_source_set_color (source, 0xBECEDD);
 		personal_source = source;
 	}
 
@@ -630,6 +641,8 @@ selector_tree_drag_motion (GtkWidget *widget,
 	
 	gtk_tree_view_set_drag_dest_row(GTK_TREE_VIEW (widget), path, GTK_TREE_VIEW_DROP_INTO_OR_BEFORE);
 	action = context->suggested_action;
+	if (action == GDK_ACTION_COPY && (context->actions & GDK_ACTION_MOVE))
+		action=GDK_ACTION_MOVE;
 
  finish:
 	if (path)
@@ -730,10 +743,10 @@ selector_tree_drag_data_received (GtkWidget *widget,
 		goto finish;
 
 	icalcomp = icalparser_parse_string (data->data);
-	
+
 	if (icalcomp) {
 		char * uid;
-
+		
 		/* FIXME deal with GDK_ACTION_ASK */
 		if (context->action == GDK_ACTION_COPY) {
 			uid = e_cal_component_gen_uid ();
@@ -745,13 +758,25 @@ selector_tree_drag_data_received (GtkWidget *widget,
 		
 		if (client) {
 			if (e_cal_open (client, TRUE, NULL)) {
-				success = TRUE;
-				update_objects (client, icalcomp);
+				icalcomponent *tmp_icalcomp = NULL;
+				GError *error = NULL;
+				uid = icalcomponent_get_uid (icalcomp);
+				if (!e_cal_get_object (client, uid, NULL, &tmp_icalcomp, &error)) {
+			        	if ((error != NULL) && (error->code != E_CALENDAR_STATUS_OBJECT_NOT_FOUND))
+						 g_message ("Failed to search the object in destination task list: %s",error->message);
+					else {
+						success = TRUE;
+						update_objects (client, icalcomp);
+					}
+
+					g_clear_error (&error);
+				} else 
+					icalcomponent_free (tmp_icalcomp);
 			}
-			
+
 			g_object_unref (client);
 		}
-		
+
 		icalcomponent_free (icalcomp);
 	}
 
