@@ -39,11 +39,13 @@
 #include <gtk/gtkvbox.h>
 
 #include <camel/camel-folder.h>
+#include <camel/camel-vee-folder.h>
 #include <libgnome/gnome-i18n.h>
 
 #include "em-folder-properties.h"
 #include "em-config.h"
 
+#include "mail-component.h"
 #include "mail-ops.h"
 #include "mail-mt.h"
 #include "mail-vfolder.h"
@@ -200,6 +202,7 @@ emfp_get_folder_item(EConfig *ec, EConfigItem *item, struct _GtkWidget *parent,
 		
 		row++;
 		l = l->next;
+		i++;
 	}
 
 	return table;
@@ -236,7 +239,14 @@ emfp_dialog_got_folder (char *uri, CamelFolder *folder, void *data)
 	camel_object_get (folder, NULL, CAMEL_FOLDER_PROPERTIES, &prop_data->properties, CAMEL_FOLDER_NAME, &prop_data->name,
 			  CAMEL_FOLDER_TOTAL, &prop_data->total, CAMEL_FOLDER_UNREAD, &prop_data->unread, NULL);
 
-	emfp_items[EMFP_FOLDER_SECTION].label = prop_data->name;
+	if (folder->parent_store == mail_component_peek_local_store(NULL)
+	    && (!strcmp(prop_data->name, "Drafts")
+		|| !strcmp(prop_data->name, "Inbox")
+		|| !strcmp(prop_data->name, "Outbox")
+		|| !strcmp(prop_data->name, "Sent")))
+		emfp_items[EMFP_FOLDER_SECTION].label = _(prop_data->name);
+	else
+		emfp_items[EMFP_FOLDER_SECTION].label = prop_data->name;
 
 	count = g_slist_length (prop_data->properties);
 
@@ -314,9 +324,23 @@ void
 em_folder_properties_show(GtkWindow *parent, CamelFolder *folder, const char *uri)
 {
 	/* HACK: its the old behaviour, not very 'neat' but it works */
-	if (!strncmp(uri, "vfolder:", 8))
-		vfolder_edit_rule(uri);
-	else if (folder == NULL)
+	if (!strncmp(uri, "vfolder:", 8)) {
+		CamelURL *url = camel_url_new(uri, NULL);
+
+		/* MORE HACK: UNMATCHED is a special folder which you can't modify, so check for it here */
+		if (url == NULL
+		    || url->fragment == NULL
+		    || strcmp(url->fragment, CAMEL_UNMATCHED_NAME) != 0) {
+			if (url)
+				camel_url_free(url);
+			vfolder_edit_rule(uri);
+			return;
+		}
+		if (url)
+			camel_url_free(url);
+	}
+
+	if (folder == NULL)
 		mail_get_folder(uri, 0, emfp_dialog_got_folder, NULL, mail_thread_new);
 	else
 		emfp_dialog_got_folder((char *)uri, folder, NULL);
