@@ -36,25 +36,27 @@
 #include <camel/camel-provider.h>
 #include <camel/camel-url.h>
 #include <camel/camel-service.h>
+#include <camel/camel-folder.h>
 #include <libedataserver/e-xml-hash-utils.h>
-#include <exchange/e2k-validate.h>
-#include <exchange/exchange-oof.h>
-#include <exchange/exchange-account.h>
 #include <e-util/e-dialog-utils.h>
+#include <e2k-validate.h>
+#include <exchange-oof.h>
+#include <exchange-account.h>
 #include "exchange-config-listener.h"
 #include "exchange-operations.h"
 #include "exchange-folder-size-display.h"
 #include "mail/em-account-editor.h"
 #include "mail/em-config.h"
 #include "exchange-delegates.h"
-
-#define ERROR_DOMAIN	"org-gnome-exchange-operations"
+#include "exchange-change-password.h"
+#include "exchange-folder-size-display.h"
 
 GtkWidget* org_gnome_exchange_settings(EPlugin *epl, EConfigHookItemFactoryData *data);
 GtkWidget *org_gnome_exchange_owa_url(EPlugin *epl, EConfigHookItemFactoryData *data);
 gboolean org_gnome_exchange_check_options(EPlugin *epl, EConfigHookPageCheckData *data);
 GtkWidget *org_gnome_exchange_auth_section (EPlugin *epl, EConfigHookItemFactoryData *data);
 void org_gnome_exchange_commit (EPlugin *epl, EConfigHookItemFactoryData *data);
+GtkWidget* org_gnome_exchange_show_folder_size_factory (EPlugin *epl, EConfigHookItemFactoryData *data); 
 
 CamelServiceAuthType camel_exchange_ntlm_authtype = {
         /* i18n: "Secure Password Authentication" is an Outlookism */
@@ -127,6 +129,10 @@ btn_chpass_clicked (GtkButton *button, gpointer data)
 		return;
 	}
 	new_password = exchange_get_new_password (old_password, TRUE);
+	if (!new_password) {
+		/* "Cacel" button was hit */
+		return;
+	}
 	g_print ("Current password is \"%s\"\n", old_password);
  	result = exchange_account_set_password (account, old_password, new_password);
 	if (result != EXCHANGE_ACCOUNT_CONNECT_SUCCESS)
@@ -151,15 +157,11 @@ btn_fsize_clicked (GtkButton *button, gpointer data)
 	GtkListStore *model;
 	GSList *acclist;
 
-	acclist = exchange_config_listener_get_accounts (exchange_global_config_listener);
-
-	/* FIXME: For now, we have only one account in the list.
-	   Find a way to handle multiple accounts.
-	*/
-	account = acclist->data; 
+	account = exchange_operations_get_exchange_account ();
 
 	model = exchange_account_folder_size_get_model (account);
-	exchange_folder_size_display (model, GTK_WIDGET (button));
+	if (model)
+		exchange_folder_size_display (model, GTK_WIDGET (button));
 }
 
 /* only used in editor */
@@ -273,8 +275,6 @@ org_gnome_exchange_settings(EPlugin *epl, EConfigHookItemFactoryData *data)
 		radio_iof = (GtkRadioButton*) gtk_object_new (GTK_TYPE_RADIO_BUTTON, "label", _("I am in the office"), NULL);
 		radio_oof = (GtkRadioButton*) gtk_object_new (GTK_TYPE_RADIO_BUTTON, "label", _("I am out of the office"), "group", radio_iof, NULL);
 	}
-	gtk_misc_set_alignment (GTK_MISC (radio_iof), 0, 0.5);
-	gtk_misc_set_alignment (GTK_MISC (radio_oof), 0, 0.5);
 	gtk_signal_connect (GTK_OBJECT (radio_oof), "toggled", G_CALLBACK (toggled_state), NULL);
 	
 
@@ -328,7 +328,6 @@ org_gnome_exchange_settings(EPlugin *epl, EConfigHookItemFactoryData *data)
 	lbl_dass = (GtkLabel*) gtk_object_new (GTK_TYPE_LABEL, "label", _("Manage the delegate settings for Exchange account"), NULL);
 	gtk_misc_set_alignment (GTK_MISC (lbl_dass), 0, 0.5);
 	btn_dass = (GtkButton*) gtk_object_new (GTK_TYPE_BUTTON, "label", _("Delegation Assitant"));
-	gtk_signal_connect (GTK_OBJECT (btn_chpass), "clicked", G_CALLBACK (btn_chpass_clicked), NULL);
 	gtk_signal_connect (GTK_OBJECT (btn_dass), "clicked", G_CALLBACK (btn_dass_clicked), NULL);
 	/* Add items to the table */
 	gtk_table_attach_defaults (tbl_auth, GTK_WIDGET (lbl_chpass), 0, 1, 0, 1);
@@ -524,12 +523,6 @@ owa_editor_entry_changed(GtkWidget *entry, EConfig *config)
 	g_free(url_string);
 }
 
-static void
-destroy_label(GtkWidget *old, GtkWidget *label)
-{
-	gtk_widget_destroy(label);
-}
-
 static char *
 construct_owa_url (CamelURL *url) 
 {
@@ -686,13 +679,9 @@ org_gnome_exchange_check_options(EPlugin *epl, EConfigHookPageCheckData *data)
 static void 
 set_oof_info ()
 {
-	GSList *accounts, *acc;
-	ExchangeAccount *account = NULL;
+	ExchangeAccount *account;
 
-	accounts = exchange_config_listener_get_accounts (exchange_global_config_listener);
-        for (acc = accounts; acc;  acc = acc->next) {
-                account = acc->data;
-        }
+	account = exchange_operations_get_exchange_account ();
 
 	if (!exchange_oof_set (account, oof_data->state, oof_data->message)) {
 
@@ -900,3 +889,44 @@ org_gnome_exchange_auth_section (EPlugin *epl, EConfigHookItemFactoryData *data)
 
 	return vbox;
 }
+
+GtkWidget *
+org_gnome_exchange_show_folder_size_factory (EPlugin *epl, EConfigHookItemFactoryData *data)
+{
+	EMConfigTargetFolder *target=  (EMConfigTargetFolder *)data->config->target;
+	CamelFolder *cml_folder = target->folder;
+	ExchangeAccount *account;
+	GtkWidget *lbl_size, *lbl_size_val;
+	GtkListStore *model;
+	GtkVBox *vbx;
+	GtkHBox *hbx_size;
+	char *folder_name, *folder_size;
+	
+	folder_name = camel_folder_get_name (cml_folder);
+	if (!folder_name)
+		folder_name = g_strdup ("name");
+	account = exchange_operations_get_exchange_account ();
+	model = exchange_account_folder_size_get_model (account);
+	if (model)
+		folder_size = g_strdup_printf ("%s KB", exchange_folder_size_get_val (model, folder_name));
+	else
+		folder_size = g_strdup ("0 KB");
+
+	hbx_size = gtk_hbox_new (FALSE, 0);
+	vbx = (GtkVBox *)gtk_notebook_get_nth_page (data->parent, 0);
+
+	lbl_size = gtk_label_new_with_mnemonic (_("Size:"));
+	lbl_size_val = gtk_label_new_with_mnemonic (_(folder_size));
+	gtk_widget_show (lbl_size);
+	gtk_widget_show (lbl_size_val);
+	gtk_misc_set_alignment (GTK_MISC (lbl_size), 0.0, 0.5);
+	gtk_misc_set_alignment (GTK_MISC (lbl_size_val), 0.0, 0.5);
+	gtk_box_pack_start (hbx_size, lbl_size, FALSE, TRUE, 12);
+	gtk_box_pack_start (hbx_size, lbl_size_val, FALSE, TRUE, 10);
+	gtk_widget_show_all (hbx_size);
+
+	gtk_box_pack_start (GTK_BOX (vbx), hbx_size, FALSE, FALSE, 0);
+	g_free (folder_size);
+
+	return hbx_size;
+}
