@@ -20,6 +20,7 @@
  */
 
 #include <string.h>
+#include <glib/gi18n.h>
 
 #include "exchange-operations.h"
 #include <e-util/e-error.h>
@@ -61,6 +62,12 @@ e_plugin_lib_enable (EPluginLib *eplib, int enable)
 	return 0;
 }
 
+ExchangeConfigListenerStatus
+exchange_is_offline (gint *mode)
+{	
+	return exchange_config_listener_get_offline_status (exchange_global_config_listener, mode);
+}	
+
 /* FIXME: See if a GLib variant of this function available */
 gboolean
 exchange_operations_tokenize_string (char **string, char *token, char delimit)
@@ -106,7 +113,7 @@ exchange_operations_cta_add_node_to_tree (GtkTreeStore *store, GtkTreeIter *pare
 
 	if (!strcmp (nodename, "personal") && !parent) {
 		/* FIXME: Don't hardcode this */
-		strcpy (nodename, "Personal Folders");
+		strcpy (nodename, _("Personal Folders"));
 	}
 
 	found = FALSE;
@@ -149,7 +156,7 @@ exchange_operations_cta_select_node_from_tree (GtkTreeStore *store, GtkTreeIter
 
 	if (!strcmp (nodename, "personal") && !parent) {
 		/* FIXME: Don't hardcode this */
-		strcpy (nodename, "Personal Folders");
+		strcpy (nodename, _("Personal Folders"));
 	}
 
 	status = gtk_tree_model_iter_children (GTK_TREE_MODEL (store), &iter, parent);
@@ -176,20 +183,33 @@ exchange_operations_cta_select_node_from_tree (GtkTreeStore *store, GtkTreeIter
 ExchangeAccount *
 exchange_operations_get_exchange_account (void) 
 {
-	ExchangeAccount *account;
+	ExchangeAccount *account = NULL;
+	ExchangeAccountResult result;
 	GSList *acclist;
 
 	acclist = exchange_config_listener_get_accounts (exchange_global_config_listener);
 	/* FIXME: Need to be changed for handling multiple accounts */
-	account = acclist->data; 
+	if (acclist) {
+		account = acclist->data; 
 	
-	return account;
+		if (exchange_account_get_context (account))
+			return account;
+		else {
+			/* Try authenticating */
+			result = exchange_config_listener_authenticate(exchange_global_config_listener, account);
+			if (exchange_account_get_context (account))
+				return account;
+		}
+	}
+	
+	return NULL;
 }
 
 void
 exchange_operations_report_error (ExchangeAccount *account, ExchangeAccountResult result)
 {
 	gchar *error_string;
+	gchar *quota_value;
 
 	g_return_if_fail (account != NULL);
 
@@ -211,9 +231,37 @@ exchange_operations_report_error (ExchangeAccount *account, ExchangeAccountResul
 		case EXCHANGE_ACCOUNT_UNKNOWN_ERROR:
 			e_error_run (NULL, error_string, account->exchange_server, NULL);
 			break;
+		case EXCHANGE_ACCOUNT_QUOTA_RECIEVE_ERROR:
+		case EXCHANGE_ACCOUNT_QUOTA_SEND_ERROR:
+		case EXCHANGE_ACCOUNT_QUOTA_WARN:
+			quota_value = g_strdup_printf ("%d", exchange_account_get_quota_limit (account));
+			e_error_run (NULL, error_string, quota_value, NULL);
+			g_free (quota_value);
+			break;
 		default:
 			e_error_run (NULL, error_string, NULL);
 	}
 	g_free (error_string);
 }
 
+void exchange_operations_update_child_esources (ESource *source, const gchar *old_path, const gchar *new_path)
+{
+	ESourceGroup *group;
+	GSList *sources, *tsource;
+	group = e_source_peek_group (source);
+	sources = e_source_group_peek_sources (group);
+	for (tsource = sources; tsource != NULL; tsource = tsource->next) {
+		gchar *ruri;
+		ruri = e_source_peek_relative_uri (tsource->data);
+		if (g_strrstr (ruri, old_path)) {
+			/* This ESource points to one of the child folders */
+			gchar **tmpv, *truri;
+			/* A nasty search and replace */
+		  	tmpv = g_strsplit (ruri, old_path, -1);
+			truri = g_strjoinv (new_path, tmpv);
+			e_source_set_relative_uri (tsource->data, truri);
+			g_strfreev (tmpv);
+			g_free (truri);	
+		}
+	}
+}
